@@ -33,6 +33,8 @@ import sip
 
 from E5Gui import E5MessageBox, E5FileDialog
 
+import Helpviewer
+
 import Preferences
 import UI.PixmapCache
 import Globals
@@ -188,6 +190,11 @@ class HelpWebPage(QWebPage):
         
         self.__adBlockedEntries = []
         self.loadStarted.connect(self.__loadStarted)
+        
+        self.saveFrameStateRequested.connect(
+            self.__saveFrameStateRequested)
+        self.restoreFrameStateRequested.connect(
+            self.__restoreFrameStateRequested)
     
     def acceptNavigationRequest(self, frame, request, type_):
         """
@@ -303,8 +310,23 @@ class HelpWebPage(QWebPage):
             urlString = bytes(info.url.toEncoded()).decode()
             errorPage.baseUrl = info.url
             if info.domain == QWebPage.QtNetwork and \
-               info.error == QNetworkReply.ContentAccessDenied and \
-               info.errorString.startswith("AdBlockRule:"):
+               info.error == QNetworkReply.ProtocolUnknownError:
+                url = QUrl(info.url)
+                res = E5MessageBox.yesNo(
+                    None,
+                    self.tr("Protocol Error"),
+                    self.tr("""Open external application for {0}-link?\n"""
+                            """URL: {1}""").format(
+                        url.scheme(), url.toString(
+                            QUrl.PrettyDecoded | QUrl.RemovePassword)),
+                    yesDefault=True)
+                
+                if res:
+                    QDesktopServices.openUrl(url)
+                return True
+            elif info.domain == QWebPage.QtNetwork and \
+                info.error == QNetworkReply.ContentAccessDenied and \
+                    info.errorString.startswith("AdBlockRule:"):
                 if info.frame != info.frame.page().mainFrame():
                     # content in <iframe>
                     docElement = info.frame.page().mainFrame()\
@@ -617,6 +639,57 @@ class HelpWebPage(QWebPage):
             return super(HelpWebPage, self).event(fakeEvent)
         
         return super(HelpWebPage, self).event(evt)
+    
+    def __saveFrameStateRequested(self, frame, itm):
+        """
+        Private slot to save the page state (i.e. zoom level and scroll
+        position).
+        
+        Note: Code is based on qutebrowser.
+        
+        @param frame frame to be saved
+        @type QWebFrame
+        @param itm web history item to be saved
+        @type QWebHistoryItem
+        """
+        try:
+            if frame != self.mainFrame():
+                return
+        except RuntimeError:
+            # With Qt 5.2.1 (Ubuntu Trusty) we get this when closing a tab:
+            #     RuntimeError: wrapped C/C++ object of type BrowserPage has
+            #     been deleted
+            # Since the information here isn't that important for closing web
+            # views anyways, we ignore this error.
+            return
+        data = {
+            'zoom': frame.zoomFactor(),
+            'scrollPos': frame.scrollPosition(),
+        }
+        itm.setUserData(data)
+    
+    def __restoreFrameStateRequested(self, frame):
+        """
+        Private slot to restore scroll position and zoom level from
+        history.
+        
+        Note: Code is based on qutebrowser.
+        
+        @param frame frame to be restored
+        @type QWebFrame
+        """
+        if frame != self.mainFrame():
+            return
+        
+        data = self.history().currentItem().userData()
+        if data is None:
+            return
+        
+        if 'zoom' in data:
+            frame.page().view().setZoomValue(int(data['zoom'] * 100))
+        
+        if 'scrollPos' in data and frame.scrollPosition() == QPoint(0, 0):
+            frame.setScrollPosition(data['scrollPos'])
 
 ###############################################################################
 
@@ -697,6 +770,9 @@ class HelpBrowser(QWebView):
         self.page().setForwardUnsupportedContent(True)
         self.page().unsupportedContent.connect(self.__unsupportedContent)
         
+        self.page().featurePermissionRequested.connect(
+            self.__featurePermissionRequested)
+        
         self.page().downloadRequested.connect(self.__downloadRequested)
         self.page().frameCreated.connect(self.__addExternalBinding)
         self.__addExternalBinding(self.page().mainFrame())
@@ -723,6 +799,8 @@ class HelpBrowser(QWebView):
         self.mw.personalInformationManager().connectPage(self.page())
         self.mw.greaseMonkeyManager().connectPage(self.page())
         
+        self.__inspector = None
+        
         self.grabGesture(Qt.PinchGesture)
     
     def __addExternalBinding(self, frame=None):
@@ -1108,12 +1186,12 @@ class HelpBrowser(QWebView):
                 UI.PixmapCache.getIcon("mailSend.png"),
                 self.tr("Send Link"),
                 self.__sendLink).setData(hit.linkUrl())
-##            if Preferences.getHelp("VirusTotalEnabled") and \
-##               Preferences.getHelp("VirusTotalServiceKey") != "":
-##                menu.addAction(
-##                    UI.PixmapCache.getIcon("virustotal.png"),
-##                    self.tr("Scan Link with VirusTotal"),
-##                    self.__virusTotal).setData(hit.linkUrl())
+            if Preferences.getHelp("VirusTotalEnabled") and \
+               Preferences.getHelp("VirusTotalServiceKey") != "":
+                menu.addAction(
+                    UI.PixmapCache.getIcon("virustotal.png"),
+                    self.tr("Scan Link with VirusTotal"),
+                    self.__virusTotal).setData(hit.linkUrl())
         
         if not hit.imageUrl().isEmpty():
             if not menu.isEmpty():
@@ -1141,12 +1219,12 @@ class HelpBrowser(QWebView):
                 UI.PixmapCache.getIcon("adBlockPlus.png"),
                 self.tr("Block Image"), self.__blockImage)\
                 .setData(hit.imageUrl().toString())
-##            if Preferences.getHelp("VirusTotalEnabled") and \
-##               Preferences.getHelp("VirusTotalServiceKey") != "":
-##                menu.addAction(
-##                    UI.PixmapCache.getIcon("virustotal.png"),
-##                    self.tr("Scan Image with VirusTotal"),
-##                    self.__virusTotal).setData(hit.imageUrl())
+            if Preferences.getHelp("VirusTotalEnabled") and \
+               Preferences.getHelp("VirusTotalServiceKey") != "":
+                menu.addAction(
+                    UI.PixmapCache.getIcon("virustotal.png"),
+                    self.tr("Scan Image with VirusTotal"),
+                    self.__virusTotal).setData(hit.imageUrl())
         
         element = hit.element()
         if not element.isNull():
@@ -1478,14 +1556,14 @@ class HelpBrowser(QWebView):
         else:
             self.__clickedMediaElement.evaluateJavaScript("this.muted = true")
     
-##    def __virusTotal(self):
-##        """
-##        Private slot to scan the selected URL with VirusTotal.
-##        """
-##        act = self.sender()
-##        url = act.data()
-##        self.mw.requestVirusTotalScan(url)
-##
+    def __virusTotal(self):
+        """
+        Private slot to scan the selected URL with VirusTotal.
+        """
+        act = self.sender()
+        url = act.data()
+        self.mw.requestVirusTotalScan(url)
+    
     def __searchRequested(self, act):
         """
         Private slot to search for some text with a selected search engine.
@@ -1628,7 +1706,25 @@ class HelpBrowser(QWebView):
         """
         Private slot to show the web inspector window.
         """
-        self.triggerPageAction(QWebPage.InspectElement)
+        if self.__inspector is None:
+            from .HelpInspector import HelpInspector
+            self.__inspector = HelpInspector()
+            self.__inspector.setPage(self.page())
+            self.__inspector.show()
+        elif self.__inspector.isVisible():
+            self.__inspector.hide()
+        else:
+            self.__inspector.show()
+    
+    def closeWebInspector(self):
+        """
+        Public slot to close the web inspector.
+        """
+        if self.__inspector is not None:
+            if self.__inspector.isVisible():
+                self.__inspector.hide()
+            self.__inspector.deleteLater()
+            self.__inspector = None
     
     def addBookmark(self):
         """
@@ -2019,6 +2115,18 @@ class HelpBrowser(QWebView):
         self.mw.historyManager().removeHistoryEntry(replyUrl, self.title())
         self.loadFinished.emit(False)
     
+    def __featurePermissionRequested(self, frame, feature):
+        """
+        Private slot handling a feature permission request.
+        
+        @param frame frame sending the request
+        @type QWebFrame
+        @param feature requested feature
+        @type QWebPage.Feature
+        """
+        manager = Helpviewer.HelpWindow.HelpWindow.featurePermissionManager()
+        manager.requestFeaturePermission(self.page(), frame, feature)
+    
     def __downloadRequested(self, request):
         """
         Private slot to handle a download request.
