@@ -16,7 +16,8 @@ from __future__ import unicode_literals
 import sys
 
 # generate list of arguments to be remembered for a restart
-restartArgsList = ["--nosplash", "--plugin", "--debug", "--config", "--pyqt4"]
+restartArgsList = ["--nosplash", "--plugin", "--debug", "--config", "--pyqt4"
+                   "--settings"]
 restartArgs = [arg for arg in sys.argv[1:]
                if arg.split("=", 1)[0] in restartArgsList]
 
@@ -24,10 +25,13 @@ import Toolbox.PyQt4ImportHook  # __IGNORE_WARNING__
 
 try:  # Only for Py2
     import StringIO as io   # __IGNORE_EXCEPTION__
-    import Utilities.compatibility_fixes     # __IGNORE_WARNING__
+    import Globals.compatibility_fixes     # __IGNORE_WARNING__
+    import locale
+    coverage = 'Python'
 except ImportError:
     import io       # __IGNORE_WARNING__
     basestring = str
+    coverage = 'Python3'
 
 try:
     import sip
@@ -53,7 +57,7 @@ except ImportError:
             sys.exit(100)
     messagebox.showerror(
         "eric6 Error",
-        "PyQt5 could not be imported. Please make sure"
+        "PyQt could not be imported. Please make sure"
         " it is installed and accessible.")
     sys.exit(100)
 
@@ -61,24 +65,34 @@ except ImportError:
 args = None
 mainWindow = None
 splash = None
+inMainLoop = None
 
 if "--debug" in sys.argv:
     del sys.argv[sys.argv.index("--debug")]
     logging.basicConfig(level=logging.DEBUG)
 
-for arg in sys.argv:
+for arg in sys.argv[:]:
     if arg.startswith("--config="):
         import Globals
         configDir = arg.replace("--config=", "")
         Globals.setConfigDir(configDir)
         sys.argv.remove(arg)
-        break
+    elif arg.startswith("--settings="):
+        from PyQt5.QtCore import QSettings
+        settingsDir = os.path.expanduser(arg.replace("--settings=", ""))
+        if not os.path.isdir(settingsDir):
+            os.makedirs(settingsDir)
+        QSettings.setPath(QSettings.IniFormat, QSettings.UserScope,
+                          settingsDir)
+        sys.argv.remove(arg)
 
 # make Third-Party package available as a packages repository
 sys.path.insert(2, os.path.join(os.path.dirname(__file__),
                                 "ThirdParty", "Pygments"))
 sys.path.insert(2, os.path.join(os.path.dirname(__file__),
                                 "ThirdParty", "Jasy"))
+sys.path.insert(2, os.path.join(os.path.dirname(__file__),
+                                "DebugClients", coverage))
 
 from E5Gui.E5Application import E5Application
 
@@ -150,6 +164,8 @@ def excepthook(excType, excValue, tracebackobj):
         traceback.print_tb(tracebackobj, None, tbinfofile)
         tbinfofile.seek(0)
         tbinfo = tbinfofile.read()
+        if sys.version_info[0] == 2:
+            tbinfo = tbinfo.decode(locale.getpreferredencoding())
     errmsg = '{0}: \n{1}'.format(str(excType), str(excValue))
     sections = [separator, timeString, separator, errmsg, separator, tbinfo]
     msg = '\n'.join(sections)
@@ -161,10 +177,20 @@ def excepthook(excType, excValue, tracebackobj):
     except IOError:
         pass
     
-    warning = str(notice) + str(msg) + str(versionInfo)
-    # Escape &<> otherwise it's not visible in the error dialog
-    warning = xml.sax.saxutils.escape(warning)
-    qWarning(warning)
+    if inMainLoop is None:
+        if sys.version_info[0] == 2:
+            notice = notice.encode(sys.stdout.encoding, 'replace')
+            msg = msg.encode(sys.stdout.encoding, 'replace')
+            versionInfo = versionInfo.encode(sys.stdout.encoding, 'replace')
+        warning = notice + msg + versionInfo
+        print(warning)
+    else:
+        warning = notice + msg + versionInfo
+        # Escape &<> otherwise it's not visible in the error dialog
+        warning = xml.sax.saxutils.escape(warning)
+        if sys.version_info[0] == 2:
+            warning = warning.encode('utf-8', 'replace')
+        qWarning(warning)
 
 
 def uiStartUp():
@@ -195,7 +221,7 @@ def main():
     from Globals import AppInfo
     import Globals
     
-    global args, mainWindow, splash, restartArgs
+    global args, mainWindow, splash, restartArgs, inMainLoop
     
     sys.excepthook = excepthook
     
@@ -208,6 +234,8 @@ def main():
          "don't open anything at startup except that given in command"),
         ("--plugin=plugin-file",
          "load the given plugin file (plugin development)"),
+        ("--settings=settingsDir",
+         "use the given directory to store the settings files"),
         ("--start-file", "load the most recently opened file"),
         ("--start-multi", "load the most recently opened multi-project"),
         ("--start-project", "load the most recently opened project"),
@@ -257,11 +285,12 @@ def main():
     if Globals.isWindowsPlatform():
         pyqtDataDir = Globals.getPyQt5ModulesDirectory()
         if os.path.exists(os.path.join(pyqtDataDir, "bin")):
-            path = os.path.join(pyqtDataDir, "bin") + \
-                os.pathsep + os.environ["PATH"]
+            path = os.path.join(pyqtDataDir, "bin")
         else:
-            path = pyqtDataDir + os.pathsep + os.environ["PATH"]
-        os.environ["PATH"] = path
+            path = pyqtDataDir
+        if sys.version_info[0] == 2:
+            path = path.encode(sys.getfilesystemencoding())
+        os.environ["PATH"] = path + os.pathsep + os.environ["PATH"]
     
     pluginFile = None
     noopen = False
@@ -291,6 +320,10 @@ def main():
     # Load translation files and install them
     loc = Startup.loadTranslators(qt4TransDir, app, ("qscintilla",))
     
+    # Initialize SSL stuff
+    from E5Network.E5SslUtilities import initSSL
+    initSSL()
+    
     splash.showMessage(QCoreApplication.translate("eric6", "Starting..."))
     # We can only import these after creating the E5Application because they
     # make Qt calls that need the E5Application to exist.
@@ -311,6 +344,7 @@ def main():
     eMsg.setMinimumSize(600, 400)
     
     # start the event loop
+    inMainLoop = True
     res = app.exec_()
     logging.debug("Shutting down, result {0:d}".format(res))
     logging.shutdown()
